import numpy as np
import os
join = os.path.join
from skimage import transform, io
from tqdm import tqdm
import torch
from segment_anything import sam_model_registry
from segment_anything.utils.transforms import ResizeLongestSide

# 1. 修改mask存放的文件夹【TODO】
gt_path = "/home/qiql/ssd/run-sam/data/SAM-dataset-551images/masks"
# 2. 修改组装好npz的保存路径【TODO】
save_path = "/home/qiql/ssd/run-sam/data/SAM-dataset-551images/npz"
# 3. 修改当前设备的GPU数量
os.environ['CUDA_VISIBLE_DEVICES'] = '0'
# 4. 当前选择的模型种类
model_type = 'vit_h'
# 5. 初始权重文件的存放路径
checkpoint = '/home/qiql/ssd/run-sam/model/sam_vit_h_4b8939.pth'
# 6. 原图image路径
img_path="/online1/ssd/qiql/run-sam/data/SAM-dataset-551images/images"

# 获取所有GT图像名称
names = sorted(os.listdir(gt_path))
os.makedirs(save_path, exist_ok=True)

device = 'cuda'
sam_model = sam_model_registry[model_type](checkpoint=checkpoint).to(device)
imgs = []
gts = []
img_embeddings = []

for gt_name in tqdm(names):
    # 如果你是jpg改一下后缀
    image_name = gt_name.split('.')[0] + ".bmp"
    print(image_name)
    # 读取GT
    gt_data = io.imread(join(gt_path, gt_name))
    # GT必须是2D，如果是3D就取前nv两通道
    if len(gt_data.shape) == 3:
        gt_data = gt_data[:, :, 0]
    assert len(gt_data.shape) == 2, 'GT must be 2D'
    # 尺寸转256数值转255
    gt_data = transform.resize(gt_data == 255, (256, 256), order=0,
                               preserve_range=True, mode='constant')
    gt_data = np.uint8(gt_data)

    # 排除GT特别小的情况，这条可以不加
    if np.sum(gt_data) > 100:
        assert np.max(gt_data) == 1 and np.unique(gt_data).shape[0] == 2, 'GT must be 2D'
        image_data = io.imread(join(img_path, image_name))
        # 计算最大值最小值
        lower_bound, upper_bound = np.percentile(image_data, 0.5), np.percentile(image_data, 99.5)
        # 排除特别特殊的像素
        image_data_pre = np.clip(image_data, lower_bound, upper_bound)
        # 归一化
        image_data_pre = (image_data_pre - np.min(image_data_pre)) / (
                np.max(image_data_pre) - np.min(image_data_pre)) * 255.0
        image_data_pre[image_data == 0] = 0
        # 归一化
        image_data_pre = transform.resize(image_data_pre, (256, 256), order=3,
                                          preserve_range=True, mode='constant', anti_aliasing=True)
        image_data_pre = np.uint8(image_data_pre)

        imgs.append(image_data_pre)
        gts.append(gt_data)
        # SAM提供的resize到1024
        sam_transform = ResizeLongestSide(sam_model.image_encoder.img_size)
        resize_img = sam_transform.apply_image(image_data_pre)
        # 要将二维灰度图像转换为三维
        if len(resize_img.shape) == 2:
            resize_img = resize_img[:, :, np.newaxis]
        # resize_img是通道在后,sam要求通道在前，transposehi是对resize_img数组进行维度重排（dimension reordering）的操作。
        resize_img_tensor = torch.as_tensor(resize_img.transpose(2, 0, 1)).to(device)
        # 增加一个channel假装当作有一个batchsize输入到sam_model.image_encoder
        input_image = sam_model.preprocess(resize_img_tensor[None, :, :, :])  # (1, 3, 1024, 1024)
        # 提前计算图像embedding
        with torch.no_grad():
            embedding = sam_model.image_encoder(input_image)
            img_embeddings.append(embedding.cpu().numpy()[0])
            
# 上面数据已经处理好并存在数组了，需要数据字典存在npz中
# 沿着纵轴堆砌，每一个都是(256, 256, 3),堆起来是(n, 256, 256, 3)
imgs = np.stack(imgs, axis=0)  # (n, 256, 256, 3)
gts = np.stack(gts, axis=0)  # (n, 256, 256)
img_embeddings = np.stack(img_embeddings, axis=0)  # (n, 1, 256, 64, 64)
# np的保存npz操作
np.savez_compressed(join(save_path, 'data.npz'), imgs=imgs, gts=gts, img_embeddings=img_embeddings)

